#!/usr/bin/env python
# Ubuntu CI Services
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import data_store
import os
import urllib2
import unittest


class TestDataStore(unittest.TestCase):

    def setUp(self):

        self.ds = data_store.DataStore("test")
        self.assert_(self.ds, "Failed to create DataStore")
        self.addCleanup(self.ds.delete, recursive=True)

        self.ds.clear()
        self.filename = 'test_file'
        self.contents = 'some data'

        super(TestDataStore, self).setUp()

    def test_list_files_empty(self):
        """ Test listing files on an empty container. """
        files = self.ds.list_files()
        self.assertEqual([], files)

    def test_add_file(self):
        """ Test adding a file. """

        self.ds.put_file(self.filename, self.contents)
        files = self.ds.list_files()
        self.assertEqual([self.filename], files)

    def test_add_file_contents(self):
        """ Test adding a file has correct contents. """

        self.ds.put_file(self.filename, self.contents)
        contents = self.ds.get_file(self.filename)

        self.assertEqual(contents, self.contents)

    def test_public_container(self):
        """ Test accesssing a public container. """

        self.ds.put_file(self.filename, self.contents)
        self.ds.change_visibility(public=True)

        response = urllib2.urlopen(self.ds.file_path(self.filename))
        self.assertEqual(response.code, 200)

    def test_delete_file(self):
        """ Test delete file removes the file. """

        self.ds.put_file(self.filename, self.contents)
        files = self.ds.list_files()
        self.assertEqual([self.filename], files)

        self.ds.delete_file(self.filename)
        files = self.ds.list_files()
        self.assertEqual([], files)

    def test_auth_missing(self):
        """ Test that missing auth details is handled correctly. """

        local_config = data_store.CONFIG_FILE
        local_config_orig = '{}.orig'.format(local_config)
        has_local_config = False

        if os.path.exists(local_config):
            has_local_config = True
            os.rename(local_config, local_config_orig)

        with self.assertRaises(data_store.DataStoreException):
            data_store.DataStore("test")

        if has_local_config:
            os.rename(local_config_orig, local_config)

if __name__ == "__main__":
    unittest.main()
