# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from os import listdir
from os.path import basename, join
from unittest import TestCase
from cli.changes_processor import ChangesProcessor
from dput.changes import Changes, ChangesFileException


TEST_DATA_ROOT = "tests/data/"


class ChangesProcessorTestCase(TestCase):

    def create_test_info(self, source_name="foobar", source_version="0.1",
                         packaging_version="1"):
        self.source_name = source_name
        self.source_version = source_version
        self.packaging_version = packaging_version
        self.changes_file = (
            "%s_%s-%s_source.changes" % (self.source_name, self.source_version,
                                         self.packaging_version))
        self.dsc_file = (
            "%s_%s-%s.dsc" % (self.source_name, self.source_version,
                              self.packaging_version))
        self.debian_tar_gz_file = (
            "%s_%s-%s.debian.tar.gz" % (self.source_name, self.source_version,
                                        self.packaging_version))
        self.orig_file = "%s_%s.orig.tar.xz" % (self.source_name,
                                                self.source_version)

    def setUp(self):
        self.create_test_info()

    def test_parse(self):
        processor = ChangesProcessor(join(TEST_DATA_ROOT, self.changes_file))
        processor.parse()
        self.assertIsInstance(processor.changes, Changes)
        self.assertEqual(processor.changes.get_package_name(),
                         self.source_name)

        self.assertEqual(self.dsc_file, processor.changes.get_dsc())
        self.assertEqual(self.debian_tar_gz_file, processor.changes.get_diff())
        self.assertTrue(self.orig_file in processor.changes.get_files())

    def test_parse_changes_file_not_found(self):
        processor = ChangesProcessor("barbaz_source.changes")
        self.assertRaises(IOError, processor.parse)

    def test_all_files_found(self):
        processor = ChangesProcessor(join(TEST_DATA_ROOT, self.changes_file))
        processor.parse()
        # Specifying where processor can find test data.
        processor.changes.set_directory(TEST_DATA_ROOT)
        self.assertEqual(len(processor.files_to_upload), 0)
        processor.validate_files()
        # The three files (dsc, debian.tar.gz and orig) plus changes file
        self.assertEqual(len(processor.files_to_upload), 4)

    def test_signature_check_fails(self):
        processor = ChangesProcessor(join(TEST_DATA_ROOT, self.changes_file))
        processor.parse()
        # Specifying where processor can find test data.
        processor.changes.set_directory(TEST_DATA_ROOT)
        self.assertRaises(ChangesFileException, processor.validate_files,
                          check_signature=True)
