# Ubuntu CI Services
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import jenkins
import os
import os.path

from restish import http, resource

from lander import utils

log = logging.getLogger(__name__)


def _get_config():
    config = None
    try:
        config_file_path = os.path.join(os.path.dirname(__file__),
                                   '../../../lander_config.json')
        if os.path.exists(config_file_path):
            with open(config_file_path) as config_file:
                config = json.loads(config_file.read())
        else:
            log.error('No lander_config.json found at: '
                      '{}'.format(os.path.abspath(config_file_path)))
    except:
        log.exception('ERROR detecting lander-jenkins config')
    return config


def _status():
    return utils.json_ok({
        'jenkins_configured': True,
    })


def _submit_request(ticket, series, source_packages, progress_trigger):
    config = _get_config()
    params = json.dumps({
        'ticket': ticket,
        'source_packages': source_packages,
        'series': series,
        'progress_trigger': progress_trigger,
    })
    params = params.replace(' ', '')
    log.debug('Params: {}'.format(params))

    jenkins_url = "http://{}:{}/".format(config['lander_hostname'],
                                         config['lander_port'])
    jenkins_job = "lander-master"
    lander_jenkins = jenkins.Jenkins(jenkins_url,
                                     config['lander_username'],
                                     config['lander_password'])
    lander_jenkins.build_job(jenkins_job,
                             {'request_id': ticket,
                              'request_parameters': params},
                             'BUILD_ME')
    r = None
    if r:
        # send only returns something if it an error message
        r = http.service_unavailable(body=r)
    return r


class API(resource.Resource):
    @resource.child()
    def status(self, request, segments):
        log.debug('[status]: %s %s', request.url, str(segments))
        return utils.http_get_resource(_status)

    @resource.child()
    def submit_request(self, request, segments):
        log.debug('[submit_request]: %s %s', request.url, str(segments))
        return utils.http_post_resource(_submit_request)
