# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from model_mommy import mommy
from ci_utils.tastypie.test import TastypieTestCase


class TicketReadAPITest(TastypieTestCase):

    def setUp(self):
        super(TicketReadAPITest, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.sourcepackage = mommy.make('SourcePackage')
        self.spu = mommy.make('SourcePackageUpload',
                              sourcepackage=self.sourcepackage)
        self.subticket = mommy.make('SubTicket', ticket=self.ticket,
                                    source_package_upload=self.spu)
        self.artifact = mommy.make('Artifact', subticket=self.subticket)

    def test_get_artifact_api(self):
        obj = self.getResource('artifact/')
        self.assertEqual(obj['objects'][0], {
            u'name': unicode(self.artifact.name),
            u'reference': unicode(self.artifact.reference),
            u'subticket': {
                u'status': self.subticket.status,
                u'current_workflow_step': self.subticket.current_workflow_step,
                u'assignee': unicode(self.subticket.assignee),
                u'source_package_upload': {
                    u'version': unicode(self.spu.version),
                    u'sourcepackage': {
                        u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                            self.sourcepackage.pk),
                        u'id': self.sourcepackage.pk,
                        u'name': unicode(self.sourcepackage.name)},
                    u'id': self.spu.pk,
                    u'resource_uri':
                    u'/api/v1/spu/{0}/'.format(self.spu.pk)},
                u'ticket': {
                    u'status': self.ticket.status,
                    u'current_workflow_step':
                    self.ticket.current_workflow_step,
                    u'description': unicode(self.ticket.description),
                    u'title': unicode(self.ticket.title),
                    u'bug_id': self.ticket.bug_id,
                    u'owner': unicode(self.ticket.owner),
                    u'base_image': unicode(self.ticket.base_image),
                    u'id': self.ticket.pk,
                    u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                        self.ticket.pk)},
                u'id': self.subticket.pk,
                u'resource_uri': u'/api/v1/subticket/{0}/'.format(
                    self.subticket.pk)},
            u'type': unicode(self.artifact.type),
            u'id': self.artifact.pk,
            u'resource_uri': u'/api/v1/artifact/{0}/'.format(self.artifact.pk),
        })

    def test_get_subticket_api(self):
        obj = self.getResource('subticket/')
        self.assertEqual(obj['objects'][0], {
            u'status': self.subticket.status,
            u'current_workflow_step': self.subticket.current_workflow_step,
            u'assignee': unicode(self.subticket.assignee),
            u'source_package_upload': {
                u'version': unicode(self.spu.version),
                u'sourcepackage': {
                    u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                        self.sourcepackage.pk),
                    u'id': self.sourcepackage.pk,
                    u'name': unicode(self.sourcepackage.name)},
                u'id': self.spu.pk,
                u'resource_uri':
                u'/api/v1/spu/{0}/'.format(self.spu.pk)},
            u'ticket': {
                u'status': self.ticket.status,
                u'current_workflow_step':
                self.ticket.current_workflow_step,
                u'description': unicode(self.ticket.description),
                u'title': unicode(self.ticket.title),
                u'bug_id': self.ticket.bug_id,
                u'owner': unicode(self.ticket.owner),
                u'base_image': unicode(self.ticket.base_image),
                u'id': self.ticket.pk,
                u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                    self.ticket.pk)},
            u'id': self.subticket.pk,
            u'resource_uri': u'/api/v1/subticket/{0}/'.format(
                self.subticket.pk),
        })

    def test_get_ticket_api(self):
        obj = self.getResource('ticket/')
        self.assertEqual(obj['objects'][0], {
            u'status': self.ticket.status,
            u'current_workflow_step':
            self.ticket.current_workflow_step,
            u'description': unicode(self.ticket.description),
            u'title': unicode(self.ticket.title),
            u'bug_id': self.ticket.bug_id,
            u'owner': unicode(self.ticket.owner),
            u'base_image': unicode(self.ticket.base_image),
            u'id': self.ticket.pk,
            u'resource_uri': u'/api/v1/ticket/{0}/'.format(
                self.ticket.pk),
        })

    def test_get_sourcepackageupload_api(self):
        obj = self.getResource('spu/')
        self.assertEqual(obj['objects'][0], {
            u'version': unicode(self.spu.version),
            u'sourcepackage': {
                u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                    self.sourcepackage.pk),
                u'id': self.sourcepackage.pk,
                u'name': unicode(self.sourcepackage.name)},
            u'id': self.spu.pk,
            u'resource_uri':
            u'/api/v1/spu/{0}/'.format(self.spu.pk),
        })
